{-
 *  Programmer:	Piotr Borek
 *  E-mail:     piotrborek@op.pl
 *  Copyright 2017 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-}
{-# LANGUAGE TemplateHaskell #-}

module Simple.UI.Widgets.Text (
    TextAlign (..),
    Text,
    TextClass,
    castToText,
    textNew,
    text,
    align
) where

import           Control.Lens                  (makeLensesFor)

import           Simple.UI.Core.Internal.UIApp

import           Simple.UI.Core.Attribute
import           Simple.UI.Widgets.Widget

data TextAlign = TextAlignLeft
               | TextAlignRight
               | TextAlignCenter

data Text = Text
    { _textParent :: Widget
    , _textText   :: Attribute (Maybe String)
    , _textAlign  :: Attribute TextAlign
    }

makeLensesFor [("_textParent", "textParent")] ''Text

class WidgetClass w => TextClass w where
    castToText :: w -> Text

    text :: w -> Attribute (Maybe String)
    text = _textText . castToText

    align :: w -> Attribute TextAlign
    align = _textAlign . castToText

instance TextClass Text where
    castToText = id

instance WidgetClass Text where
    castToWidget = _textParent

    overrideWidget = overrideWidgetHelper textParent

textNew :: Maybe String -> UIApp u Text
textNew s = do
    parent <- widgetNew

    t <- attributeNew s
    a <- attributeNew TextAlignLeft
    return Text
        { _textParent = parent
        , _textText = t
        , _textAlign = a
        }
